/***************************************************************************
 *         GSkyRegion.hpp - Abstract virtual sky region base class         *
 * ----------------------------------------------------------------------- *
 *  copyright (C) 2009-2013 by Juergen Knoedlseder                         *
 * ----------------------------------------------------------------------- *
 *                                                                         *
 *  This program is free software: you can redistribute it and/or modify   *
 *  it under the terms of the GNU General Public License as published by   *
 *  the Free Software Foundation, either version 3 of the License, or      *
 *  (at your option) any later version.                                    *
 *                                                                         *
 *  This program is distributed in the hope that it will be useful,        *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of         *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *  GNU General Public License for more details.                           *
 *                                                                         *
 *  You should have received a copy of the GNU General Public License      *
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.  *
 *                                                                         *
 ***************************************************************************/
/**
 * @file GSkyRegion.hpp
 * @brief Abstract sky region base class interface definition
 * @author Pierrick Martin
 */

#ifndef GSKYREGION_HPP
#define GSKYREGION_HPP

/* __ Includes ___________________________________________________________ */
#include <vector>
#include <string>
#include "GBase.hpp"
#include "GXmlElement.hpp"


/***********************************************************************//**
 * @class GSkyRegion
 *
 * @brief Abstract interface for the sky region class
 *
 * This class provides an abstract interface for a sky region. The sky region 
 * is defined by an array of parameters, the meaning of which is specific 
 * to the derived class where the region type or shape is defined. 
 * The class holds several properties such as solid angle subtended by the 
 * region ... etc
 *
 ***************************************************************************/
class GSkyRegion : public GBase {

public:
    // Constructors and destructors
    GSkyRegion(void);
    GSkyRegion(const GSkyRegion& region);
    explicit GSkyRegion(const GXmlElement& xml);  
    virtual ~GSkyRegion(void);

    // Operators
    virtual GSkyRegion&   operator=(const GSkyRegion& region);
    virtual double  operator[](const int& index);
    virtual const double  operator[](const int& index) const;
    virtual double  operator[](const std::string& name);
    virtual const double  operator[](const std::string& name) const;

    // Pure virtual methods
    virtual void         clear(void) = 0;
    virtual GSkyRegion*  clone(void) const = 0;
    virtual std::string  type(void) const = 0;
    virtual void         read(const GXmlElement& xml) = 0;
    virtual void         write(GXmlElement& xml) const = 0;
    virtual std::string  print(const GChatter& chatter = NORMAL) const = 0;

    // Implemented methods
    int                 size(void) const;
    const double        at(const int& index) const;
    bool                haspar(const std::string& name) const;
    const std::string&  name(void) const;
    void                name(const std::string& name);
    std::string         ids(void) const;
    void                ids(const std::string& ids);

protected:
    // Protected methods
    void         init_members(void);
    void         copy_members(const GSkyRegion& region);
    void         free_members(void);
    std::string  print_attributes(void) const;

    // Protected members
    std::string                  m_name;         //!< Model namespace
    std::vector<std::string>     m_ids;          //!< Identifiers to which region applies
    std::vector<double>          m_pars;         //!< Pointers to all region parameters
};


/***********************************************************************//**
 * @brief Returns reference to region parameter by index
 *
 * @param[in] index Parameter index [0,...,size()-1].
 * @return Reference to region parameter.
 *
 * Returns a reference to the region parameter of the specified @p index.
 ***************************************************************************/
inline
double GSkyRegion::operator[](const int& index)
{
    // Return reference
    return *(m_pars[index]);
}


/***********************************************************************//**
 * @brief Returns reference to region parameter by index (const version)
 *
 * @param[in] index Parameter index [0,...,size()-1].
 * @return Const reference to region parameter.
 *
 * Returns a const reference to the region parameter of the specified
 ***************************************************************************/
inline
const double GSkyRegion::operator[](const int& index) const
{
    // Return reference
    return *(m_pars[index]);
}


/***********************************************************************//**
 * @brief Return number of parameters in region
 *
 * @return Number of parameters in region.
 *
 * Returns the number of parameters in the region.
 ***************************************************************************/
inline
int GSkyRegion::size(void) const
{
    return (m_pars.size());
}


/***********************************************************************//**
 * @brief Return parameter name
 *
 * @return Parameter name.
 *
 * Returns the parameter name.
 ***************************************************************************/
inline
const std::string& GSkyRegion::name(void) const
{
    return m_name;
}


/***********************************************************************//**
 * @brief Set parameter name
 *
 * @param[in] name Parameter name.
 *
 * Set the parameter name.
 ***************************************************************************/
inline
void GSkyRegion::name(const std::string& name)
{
    m_name = name;
    return;
}

#endif /* GSKYREGION_HPP */